@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2006 Nokia Corporation.
+ * Copyright (C) 2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +29,7 @@
 
 G_BEGIN_DECLS
 
+#include <libintl.h>
 #include <osso-log.h>
 
 #ifdef HAVE_CONFIG_H
@@ -39,10 +39,19 @@ G_BEGIN_DECLS
 #define _(String) dgettext(PACKAGE, String)
 #define N_(String) String
 #define HCS(String) dgettext("hildon-common-strings", String)
+#define KE(String) dgettext("ke-recv", String)
 
 /* If environment doesn't define, use this */
 #define MAX_FILENAME_LENGTH_DEFAULT 255
 
+/* Default weights for sorting operation. Negative weight informs
+   that only single sorting criteria (=name) is used. */
+#define SORT_WEIGHT_FILE   10
+#define SORT_WEIGHT_FOLDER -10
+#define SORT_WEIGHT_DEVICE -5
+#define SORT_WEIGHT_INTERNAL_MMC -8
+#define SORT_WEIGHT_EXTERNAL_MMC -7
+
 /* An easy way to add tracing to functions, used while debugging */
 #if 0
 #define TRACE ULOG_DEBUG_F("entered")
