@@ -732,7 +732,7 @@ void hildon_file_details_dialog_set_file_iter(HildonFileDetailsDialog *self, Gtk
     -1);
 
   g_object_set(self->priv->file_name, "label", name, NULL);
-  g_object_set(self->priv->file_type, "label", _(mime), NULL);
+  g_object_set(self->priv->file_type, "label", mime, NULL);
 
   if (size < 1024)
     g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_kb"), 1);
