@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,12 +36,11 @@ struct _HildonFileSystemInfo
   GtkFileSystem *fs;
   GtkFilePath *path;  
   GtkFileInfo *info;
-  HildonFileSystemModelItemType type;
+  HildonFileSystemSpecialLocation *location;
 
   gchar *name_cache;
   GdkPixbuf *icon_cache;
   gint size;
-  gboolean mmc2;
 };
 
 /**
@@ -62,14 +60,12 @@ hildon_file_system_info_new(const gchar *uri, GError **error)
     GtkFileSystem *fs;
     GtkFilePath *path, *parent_path;
     HildonFileSystemInfo *result;
-    HildonFileSystemModelItemType type;
+    HildonFileSystemSpecialLocation *location;
     GtkFileInfo *info = NULL;
     GtkFileFolder *folder;
-    gboolean mmc2 = FALSE;
 
     g_return_val_if_fail(uri != NULL, NULL);
 
-    _hildon_file_system_ensure_locations();
     fs = hildon_file_system_create_backend("gnome-vfs", TRUE);
     g_return_val_if_fail(GTK_IS_FILE_SYSTEM(fs), NULL);
 
@@ -89,59 +85,39 @@ hildon_file_system_info_new(const gchar *uri, GError **error)
       return NULL;
     }
 
-    /* We found toplevel item */
-    if (!parent_path)
+    location = _hildon_file_system_get_special_location(fs, path);
+
+    if (!location)
     {
-      if (g_str_has_prefix(uri, "obex://"))
-        type = HILDON_FILE_SYSTEM_MODEL_GATEWAY;
-      else
-      {
-        info = gtk_file_info_new();
-        gtk_file_info_set_is_folder(info, TRUE);
-        gtk_file_info_set_display_name(info, gtk_file_path_get_string(path));
-        type = HILDON_FILE_SYSTEM_MODEL_FOLDER;
-      }
-    }
-    else
-    {
-      type = _hildon_file_system_get_special_location(fs, path);
-
-      if (type == -1)
-      {
-        mmc2 = TRUE;
-        type = HILDON_FILE_SYSTEM_MODEL_MMC;
-      }
-
-      if (!type)
-      {
-        folder = gtk_file_system_get_folder(fs, parent_path, GTK_FILE_INFO_ALL, error);
-        if (folder)
-        {
-          info = gtk_file_folder_get_info(folder, path, error);
-          g_object_unref(folder);
+        if (parent_path) {
+            folder = gtk_file_system_get_folder(fs, parent_path, 
+                        GTK_FILE_INFO_ALL, error);
+            if (folder)
+            {
+                info = gtk_file_folder_get_info(folder, path, error);
+                g_object_unref(folder);
+            }
+            if (!info)
+            {
+              g_object_unref(fs);
+              gtk_file_path_free(parent_path);
+              gtk_file_path_free(path);
+              return NULL;
+            }
+        } else {
+            info = gtk_file_info_new();
+            gtk_file_info_set_is_folder(info, TRUE);
+            gtk_file_info_set_display_name(info, gtk_file_path_get_string(path));
         }
-        if (!info)
-        {
-          g_object_unref(fs);
-          gtk_file_path_free(parent_path);
-          gtk_file_path_free(path);
-          return NULL;
-        }
-
-        type = gtk_file_info_get_is_folder(info) ? 
-                 HILDON_FILE_SYSTEM_MODEL_FOLDER :
-                 HILDON_FILE_SYSTEM_MODEL_FILE;  
-      }
-
-      gtk_file_path_free(parent_path);
     }
 
+    gtk_file_path_free(parent_path);
+
     result = g_new0(HildonFileSystemInfo, 1);
     result->fs = fs;
     result->path = path;
     result->info = info;
-    result->type = type;
-    result->mmc2 = mmc2;
+    result->location = location;
 
     return result;    
 }
@@ -306,7 +282,7 @@ hildon_file_system_info_get_display_name(HildonFileSystemInfo *info)
 
   if (info->name_cache == NULL)
     info->name_cache = _hildon_file_system_create_display_name(info->fs, 
-        info->path, info->type, info->info, info->mmc2);
+        info->path, info->location, info->info);
 
   return info->name_cache;
 }
@@ -358,7 +334,7 @@ hildon_file_system_info_get_icon_at_size(HildonFileSystemInfo *info,
   if (info->icon_cache == NULL)
     info->icon_cache = _hildon_file_system_create_image(info->fs,  
                   ref_widget, info->path, 
-                  info->type, size, info->mmc2);
+                  info->location, size);
 
   info->size = size;
 
@@ -380,6 +356,8 @@ void hildon_file_system_info_free(HildonFileSystemInfo *info)
     gtk_file_info_free(info->info);
   if (info->icon_cache)
     g_object_unref(info->icon_cache);
+  if (info->location)
+    g_object_unref(info->location);
 
   gtk_file_path_free(info->path);
   g_free(info->name_cache);
