@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -72,6 +71,10 @@ typedef enum {
     HILDON_FILE_SYSTEM_MODEL_COLUMN_AUTHOR,
 
     HILDON_FILE_SYSTEM_MODEL_COLUMN_IS_HIDDEN,
+    HILDON_FILE_SYSTEM_MODEL_COLUMN_UNAVAILABLE_REASON,
+    HILDON_FILE_SYSTEM_MODEL_COLUMN_FAILED_ACCESS_MESSAGE,
+    HILDON_FILE_SYSTEM_MODEL_COLUMN_SORT_WEIGHT,
+    HILDON_FILE_SYSTEM_MODEL_COLUMN_EXTRA_INFO,
 
     HILDON_FILE_SYSTEM_MODEL_NUM_COLUMNS
 } HildonFileSystemModelColumns;
@@ -103,13 +106,15 @@ struct _HildonFileSystemModel {
     HildonFileSystemModelPrivate *priv;
 };
 
-typedef struct {
+typedef struct _HildonFileSystemModelClass HildonFileSystemModelClass;
+
+struct _HildonFileSystemModelClass {
     GObjectClass parent_class;
 
     void (*finished_loading) (HildonFileSystemModel *model, GtkTreeIter *iter);
     void (*device_disconnected) (HildonFileSystemModel *model, 
       GtkTreeIter *iter);
-} HildonFileSystemModelClass;
+};
 
 GType hildon_file_system_model_get_type(void);
 
@@ -170,8 +175,6 @@ void hildon_file_system_model_reset_available(HildonFileSystemModel *model);
 
 void _hildon_file_system_model_queue_reload(HildonFileSystemModel *model,
   GtkTreeIter *parent_iter, gboolean force);
-void _hildon_file_system_model_load_children(HildonFileSystemModel *model,
-  GtkTreeIter *parent_iter);
 
 GtkFileSystem
     *_hildon_file_system_model_get_file_system(HildonFileSystemModel *
