@@ -42,11 +42,13 @@
 #include <libintl.h>
 #include <string.h>
 
+#include <gdk/gdkx.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#include <osso-ic-dbus.h>
 
 #include "hildon-file-common-private.h"
 #include "hildon-file-system-settings.h"
@@ -61,7 +63,9 @@ enum {
   PROP_MMC_COVER_OPEN,
   PROP_MMC_USED,
   PROP_MMC_CORRUPTED,
-  PROP_INTERNAL_MMC_CORRUPTED
+  PROP_INTERNAL_MMC_CORRUPTED,
+  PROP_IAP_CONNECTED,
+  PROP_BONDING_CHANGED
 };
 
 #define PRIVATE(obj) HILDON_FILE_SYSTEM_SETTINGS(obj)->priv
@@ -79,6 +83,17 @@ enum {
 #define MCE_MATCH_RULE "type='signal',interface='" MCE_SIGNAL_IF \
                        "',member='" MCE_DEVICE_MODE_SIG "'"
 
+#define ICD_MATCH_RULE "type='signal',interface='" ICD_DBUS_INTERFACE \
+                       "',member='" ICD_STATUS_CHANGED_SIG "'"
+
+#define BT_BONDING_CREATED_RULE "type='signal',"                 \
+                                "interface='org.bluez.Adapter'," \
+                                "member='BondingCreated'"
+
+#define BT_BONDING_REMOVED_RULE "type='signal',"                 \
+                                "interface='org.bluez.Adapter'," \
+                                "member='BondingRemoved'"
+
 /* For getting and tracking the Bluetooth name
  */
 #define BTNAME_SERVICE                  "org.bluez"
@@ -104,6 +119,7 @@ struct _HildonFileSystemSettingsPrivate
   gchar *btname;
   gchar *gateway;
   gboolean gateway_ftp;
+  gboolean iap_connected;
 
   gboolean gconf_ready;
   gboolean flightmode_ready;
@@ -155,6 +171,12 @@ hildon_file_system_settings_get_property(GObject *object,
     case PROP_MMC_CORRUPTED:
       g_value_set_boolean(value, priv->mmc_is_corrupted);
       break;
+    case PROP_IAP_CONNECTED:
+      g_value_set_boolean(value, priv->iap_connected);
+      break;
+    case PROP_BONDING_CHANGED:
+      g_value_set_boolean(value, TRUE);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -327,6 +349,33 @@ set_flight_mode_from_message(HildonFileSystemSettings *self,
   }
 }
 
+static void 
+set_icd_status_from_message (HildonFileSystemSettings *self,
+                             DBusMessage *message)
+{
+  gchar *name, *type, *status, *uierr;
+  gboolean new_value = self->priv->iap_connected;
+
+  if (dbus_message_get_args (message, NULL,
+			     DBUS_TYPE_STRING, &name,
+			     DBUS_TYPE_STRING, &type,
+			     DBUS_TYPE_STRING, &status,
+			     DBUS_TYPE_STRING, &uierr,
+			     DBUS_TYPE_INVALID))
+    {
+      if (strcmp (status, "IDLE") == 0)
+	new_value = FALSE;
+      else if (strcmp (status, "CONNECTED") == 0)
+	new_value = TRUE;
+    }
+
+  if (new_value != self->priv->iap_connected)
+    {
+      self->priv->iap_connected = new_value;
+      g_object_notify (G_OBJECT(self), "iap-connected");
+    }
+}
+
 static DBusHandlerResult
 hildon_file_system_settings_handle_dbus_signal(DBusConnection *conn,
                                                DBusMessage *msg,
@@ -335,14 +384,25 @@ hildon_file_system_settings_handle_dbus_signal(DBusConnection *conn,
   g_assert(HILDON_IS_FILE_SYSTEM_SETTINGS(data));
 
   if (dbus_message_is_signal(msg, MCE_SIGNAL_IF, MCE_DEVICE_MODE_SIG))
-  {
-    set_flight_mode_from_message(HILDON_FILE_SYSTEM_SETTINGS(data), msg);
-  }
+    {
+      set_flight_mode_from_message(HILDON_FILE_SYSTEM_SETTINGS(data), msg);
+    }
   else if (dbus_message_is_signal(msg, BTNAME_SIGNAL_IF, BTNAME_SIG_CHANGED))
-  {
-    set_bt_name_from_message(HILDON_FILE_SYSTEM_SETTINGS(data), msg);
-  }
-
+    {
+      set_bt_name_from_message(HILDON_FILE_SYSTEM_SETTINGS(data), msg);
+    }
+  else if (dbus_message_is_signal(msg, "org.bluez.Adapter", "BondingCreated")
+	   || dbus_message_is_signal(msg, "org.bluez.Adapter",
+				     "BondingRemoved"))
+    {
+      g_object_notify (data, "bonding-changed");
+    }
+  else if (dbus_message_is_signal (msg, ICD_DBUS_INTERFACE,
+				   ICD_STATUS_CHANGED_SIG))
+    {
+      set_icd_status_from_message (HILDON_FILE_SYSTEM_SETTINGS (data), msg);
+    }
+				   
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
@@ -465,6 +525,7 @@ hildon_file_system_settings_setup_dbus(HildonFileSystemSettings *self)
   dbus_message_unref(request);
 
   dbus_connection_setup_with_g_main(conn, NULL);
+  dbus_error_init (&error);
   dbus_bus_add_match(conn, MCE_MATCH_RULE, &error);
   if (dbus_error_is_set(&error))
   {
@@ -472,6 +533,7 @@ hildon_file_system_settings_setup_dbus(HildonFileSystemSettings *self)
     dbus_error_free(&error);
   }
 
+  dbus_error_init (&error);
   dbus_bus_add_match(conn, BTNAME_MATCH_RULE, &error);
   if (dbus_error_is_set(&error))
   {
@@ -479,6 +541,30 @@ hildon_file_system_settings_setup_dbus(HildonFileSystemSettings *self)
     dbus_error_free(&error);
   }
 
+  dbus_error_init (&error);
+  dbus_bus_add_match (conn, ICD_MATCH_RULE, &error);
+  if (dbus_error_is_set(&error))
+    {
+      ULOG_ERR_F("dbus_bus_add_match failed: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+
+  dbus_error_init (&error);
+  dbus_bus_add_match (conn, BT_BONDING_CREATED_RULE, &error);
+  if (dbus_error_is_set(&error))
+    {
+      ULOG_ERR_F("dbus_bus_add_match failed: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+
+  dbus_error_init (&error);
+  dbus_bus_add_match (conn, BT_BONDING_REMOVED_RULE, &error);
+  if (dbus_error_is_set(&error))
+    {
+      ULOG_ERR_F("dbus_bus_add_match failed: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+
   if (!dbus_connection_add_filter(conn,
       hildon_file_system_settings_handle_dbus_signal, self, NULL))
   {
@@ -546,6 +632,7 @@ hildon_file_system_settings_finalize(GObject *obj)
   {
     dbus_bus_remove_match(priv->dbus_conn, MCE_MATCH_RULE, NULL);
     dbus_bus_remove_match(priv->dbus_conn, BTNAME_MATCH_RULE, NULL);
+    dbus_bus_remove_match(priv->dbus_conn, ICD_MATCH_RULE, NULL);
     dbus_connection_remove_filter(priv->dbus_conn,
       hildon_file_system_settings_handle_dbus_signal, obj);
     dbus_connection_unref(priv->dbus_conn);
@@ -603,6 +690,14 @@ hildon_file_system_settings_class_init(HildonFileSystemSettingsClass *klass)
     g_param_spec_boolean("mmc-cover-open", "MMC cover open",
 	                 "Whether or not the MMC cover is open",
 			 FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_IAP_CONNECTED,
+    g_param_spec_boolean("iap-connected", "IAP Connected",
+	                 "Whether or not we have a internet connection",
+			 FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_BONDING_CHANGED,
+    g_param_spec_boolean("bonding-changed", "Bluetooth bondings changed",
+	                 "Hack: only used for notify signals...",
+			 FALSE, G_PARAM_READABLE));
 }
 
 static gboolean delayed_init(gpointer data)
@@ -726,6 +821,7 @@ hildon_file_system_settings_init(HildonFileSystemSettings *self)
     HILDON_TYPE_FILE_SYSTEM_SETTINGS, HildonFileSystemSettingsPrivate); 
  
   self->priv->flightmode = TRUE;
+  self->priv->iap_connected = TRUE;
 
   /* This ugly stuff blocks the execution, so let's do it
      only after we are in idle */
@@ -788,6 +884,37 @@ static dbus_int32_t banner_pid = 0;
 #define BANNER_HIDE "cancel_delayed_infobanner"
 #define BANNER_TIMEOUT 500
 
+/* We need to make the updating banner transient to a toplevel window
+   but the HildonFileSystemModel knows nothing about the widget that
+   it is displayed in.  Thus the HildonFileSelection widget informs us
+   when it is displayed.
+*/
+
+static GtkWindow *banner_toplevel_window = NULL;
+
+static void
+banner_toplevel_window_destroy (GtkWindow *win, gpointer unused)
+{
+  if (banner_toplevel_window == win)
+    {
+      banner_toplevel_window = NULL;
+      g_object_unref (win);
+    }
+}
+
+void
+_hildon_file_system_set_banner_toplevel (GtkWindow *win)
+{
+  if (banner_toplevel_window == NULL)
+    {
+      g_object_ref (win);
+      banner_toplevel_window = win;
+      g_signal_connect (win, "destroy",
+			G_CALLBACK (banner_toplevel_window_destroy),
+			NULL);
+    }
+}
+
 /* Communication with tasknavigator for displaying possible
    banner while making blocking calls */
 void _hildon_file_system_prepare_banner(guint *timeout_id)
@@ -798,8 +925,16 @@ void _hildon_file_system_prepare_banner(guint *timeout_id)
   static const dbus_int32_t initial_value = 1000;
   static const dbus_int32_t display_timeout = 30000;
   const char *ckdg_pb_updating_str;
+  dbus_int32_t xid = 0;
   dbus_bool_t ret;
 
+  /* We don't show the banner if there is no toplevel window for it
+     yet.
+  */
+  if (banner_toplevel_window == NULL
+      || GTK_WIDGET (banner_toplevel_window)->window == NULL)
+    return;
+
   if (timeout_id == NULL) return;
 
   /* just refresh the timeout if there is already a timeout */
@@ -831,10 +966,13 @@ void _hildon_file_system_prepare_banner(guint *timeout_id)
   if (banner_pid == 0)
     banner_pid = getpid(); /* id */
 
+  xid = GDK_WINDOW_XID (GTK_WIDGET (banner_toplevel_window)->window);
+
   ret = dbus_message_append_args(message, DBUS_TYPE_INT32, &banner_pid,
                                  DBUS_TYPE_INT32, &initial_value,
                                  DBUS_TYPE_INT32, &display_timeout,
                                  DBUS_TYPE_STRING, &ckdg_pb_updating_str,
+                                 DBUS_TYPE_INT32, &xid,
                                  DBUS_TYPE_INVALID);
   if (!ret)
   {
