@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,6 +36,7 @@
 #define __HILDON_FILE_SYSTEM_SETTINGS_H__
 
 #include <glib-object.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -76,13 +76,11 @@ HildonFileSystemSettings *_hildon_file_system_settings_get_instance(void);
 /* Returns TRUE, if async queries have finished */
 gboolean _hildon_file_system_settings_ready(HildonFileSystemSettings *self);
 
-/* Convenience APIs */
-gboolean _hildon_file_system_settings_get_flight_mode(void);
-
 /* Communication with tasknavigator for displaying possible
    banner while making blocking calls */
-void _hildon_file_system_prepare_banner(void);
-void _hildon_file_system_cancel_banner(void);
+void _hildon_file_system_prepare_banner(guint *timeout_id);
+void _hildon_file_system_cancel_banner(guint *timeout_id);
+void _hildon_file_system_set_banner_toplevel (GtkWindow *win);
 
 G_END_DECLS
 
